/* main.c - Application main entry point */


#include <zephyr/types.h>
#include <stddef.h>
#include <zephyr/sys/util.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/pm/device.h>
#include <stdio.h>
#include <string.h>


#include <version.h>
#include <ncs_version.h>

#include <hal/nrf_gpio.h>
#include <nrfx_timer.h>
#include <nrfx_rtc.h>



#include "freq_meter.h"
#include "main.h"
#include "compile_timestamp.h"



#define RTC_INSTANCE 2
#define CC_CHANNEL 0
#define RTC_TIMEOUT_MS 1000
#define RTC_PRESCALER_MS 125


#define DEBUG_PRINT 1

nrfx_rtc_t rtc = NRFX_RTC_INSTANCE(RTC_INSTANCE);

nrfx_err_t nrfx_err;
volatile uint32_t timeout = 0;
volatile uint32_t tilt_counter = 0;
volatile uint32_t in_counter = 0;
volatile uint32_t total_tilt_counter = 0;
volatile uint32_t total_in_counter = 0;

uint8_t rtc_initialized = 0;
nrfx_rtc_config_t rtc_config = NRFX_RTC_DEFAULT_CONFIG;

typedef void (*tilt_irq_handler_t)(void);
tilt_irq_handler_t tilt_handler;
typedef void (*in_irq_handler_t)(void);
in_irq_handler_t in_handler;

char str[128];

const struct device *const cons = DEVICE_DT_GET(DT_CHOSEN(zephyr_console));

void my_printk(uint8_t *data)
{
	
		enum pm_device_state pm_state;    
		pm_device_state_get(cons, &pm_state);
		switch(pm_state) {
			case PM_DEVICE_STATE_ACTIVE:
				break;
			case PM_DEVICE_STATE_SUSPENDED:
				pm_device_action_run(cons, PM_DEVICE_ACTION_RESUME);
				break;
			case PM_DEVICE_STATE_SUSPENDING:
				break;
			case PM_DEVICE_STATE_OFF:
				break;
		}
		printf("%s", data);
		pm_device_action_run(cons, PM_DEVICE_ACTION_SUSPEND);
	
}


struct gpio_callback int1_tilt_cb_data;
struct gpio_callback int1_in_cb_data;


static void my_gpio_callback1(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	// Obsługa zdarzenia niskiego stanu GPIO
	//my_printk("Low state detected on GPIO pin int1 !\n");
}

struct gpio_callback my_gpio_cb1, my_gpio_cb2;

void rtc_event_handler(nrfx_rtc_int_type_t int_type)
{
	switch (int_type)
	{
	case NRFX_RTC_INT_COMPARE0:
		// printk("RTC compare event occurred\n");
		nrfx_rtc_counter_clear(&rtc);
		nrfx_rtc_cc_set(&rtc, CC_CHANNEL, RTC_TIMEOUT_MS / RTC_PRESCALER_MS, true);
		tilt_counter = nrfx_timer_capture(&timer1, 0);
		total_tilt_counter += tilt_counter;
		in_counter = nrfx_timer_capture(&timer2, 0);
		total_in_counter += in_counter;
		if (tilt_counter > 0)
		{
			timeout = 15;
			// my_printk("Tilt counter detected - no sleep\r\n");
		}
		if (in_counter > 0)
		{
			timeout = 15;
			// my_printk("In counter detected - no sleep\r\n");
		}
		nrfx_timer_clear(&timer1);
		nrfx_timer_clear(&timer2);
		break;
	default:
		break;
	}
}

void counter_timers_init(void)
{	
	rtc_config.prescaler = NRF_RTC_FREQ_TO_PRESCALER(1000 / RTC_PRESCALER_MS);
	nrfx_rtc_init(&rtc, &rtc_config, rtc_event_handler);

	IRQ_CONNECT(DT_IRQN(DT_NODELABEL(rtc2)),
				DT_IRQ(DT_NODELABEL(rtc2), priority),
				nrfx_isr, nrfx_rtc_2_irq_handler, 0);
	irq_enable(DT_IRQN(DT_NODELABEL(rtc2)));

	nrfx_rtc_tick_disable(&rtc);
	nrfx_rtc_counter_clear(&rtc);
	nrfx_rtc_enable(&rtc);
	nrfx_rtc_cc_set(&rtc, CC_CHANNEL, RTC_TIMEOUT_MS / RTC_PRESCALER_MS, true);

	nrf_gpio_cfg_input(int1_tilt.pin, NRF_GPIO_PIN_NOPULL);
	nrf_gpio_cfg_input(int1_in.pin, NRF_GPIO_PIN_NOPULL);

	counter1_init();
	counter2_init();
	gpiote_init(int1_tilt.pin, int1_in.pin);

	ppi_timer_start_counter_init();
	ppi_gpiote_counter_init();

	rtc_initialized = 1;
}


int main(void)
{

	int err;
	int ret;
	uint8_t *buf;

#if DEBUG_PRINT
	sprintf(str, "\r\n\r\n\033[32mStarting %s: ...\033[0m\r\n", CONFIG_BOARD);
	my_printk(str);
	sprintf(str,"Zephyr version: %s\r\n", KERNEL_VERSION_STRING);
	my_printk(str);
	sprintf(str,"NCS version: %s\r\n", NCS_VERSION_STRING);
	my_printk(str);
#endif

#if DEBUG_PRINT
	sprintf(str, "Compiled at: %s %s\r\n", __DATE__, __TIME__);
	my_printk(str);
#endif
	counter_timers_init();

	do
	{

			sprintf(str, "Tilt counter: %d\r\n", tilt_counter);
			my_printk(str);
			sprintf(str, "In counter: %d\r\n", in_counter);
			my_printk(str);

			

		k_sleep(K_MSEC(1000));
	} while (1);


	return 0;
}
